/*
 * $Id: cilk-conf.h,v 1.3 2001/01/22 23:11:34 matley Exp $
 */
/*
 * Copyright (c) 2000 Massachusetts Institute of Technology
 * Copyright (c) 2000 Matteo Frigo
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */

#ifndef _CILK_CONF_H
#define _CILK_CONF_H

/* use the __cilksafe__ attribute if running through Cilk */
#ifdef __CILK__
#define __CILKSAFE__  __cilksafe__
#else
#define __CILKSAFE__
#endif

/*
 * Keep track of various statistics
 */
#ifndef CILK_STATS
#define CILK_STATS 0
#endif

/*
 * Keep track of execution time
 */
#ifndef CILK_TIMING
#define CILK_TIMING 0
#endif

/*
 * Measure critical path. EXPENSIVE in terms of execution time
 */
#ifndef CILK_CRITICAL_PATH
#define CILK_CRITICAL_PATH 0
#endif

/*
 * Provide the current work and critical path in
 * Cilk_user_work and Cilk_user_critical_path
 */
#define CILK_USER_TIMERS CILK_CRITICAL_PATH

/*
 * Enable rts self-consistency checks.
 */
#ifndef CILK_DEBUG
#define CILK_DEBUG 0
#endif

/*
 * enable support for alloca (slows down common case frame
 * allocation by 20%)
 */
#ifndef CILK_ALLOCA
#define CILK_ALLOCA 1
#endif

/* 
 * Default size of Cilk's stack
 */
#define CILK_DEFAULT_STACK_DEPTH 32768  

/*
 * there should be no need to hack below this line
 */
#if CILK_CRITICAL_PATH && (!CILK_TIMING)
#warning Critical path measurements cannot be enabled because timing is not supported by the rts
#undef CILK_CRITICAL_PATH
#define CILK_CRITICAL_PATH 0
#endif

#define CILK_CHECK(cond, complaints) \
     if (!(cond)) Cilk_die_internal complaints;

#if CILK_DEBUG
#define CILK_ASSERT(ws, ex)						 \
      (void)((ex) || 							 \
	     (Cilk_die_internal(ws, USE_PARAMETER(assertion_failed_msg), \
             #ex, __LINE__, __FILE__), 0))
#define WHEN_CILK_DEBUG(ex)  ex
#define CILK_COMPLAIN(cond, complaints) \
     if (!(cond)) Cilk_die_internal complaints;
#else
#define CILK_ASSERT(ws, ex)
#define WHEN_CILK_DEBUG(ex)
#define CILK_COMPLAIN(cond, complaints) 
#endif

#if CILK_ALLOCA
#define WHEN_CILK_ALLOCA(a) a
#else
#define WHEN_CILK_ALLOCA(a)
#endif

#if CILK_STATS
#define WHEN_CILK_STATS(a) a
#else
#define WHEN_CILK_STATS(a)
#endif

#if CILK_TIMING
#define WHEN_CILK_TIMING(a) a
#else
#define WHEN_CILK_TIMING(a)
#endif

#if CILK_CRITICAL_PATH
#define WHEN_CILK_CRITICAL_PATH(a) a
#else
#define WHEN_CILK_CRITICAL_PATH(a)
#endif

#if CILK_USER_TIMERS
#define WHEN_CILK_USER_TIMERS(a) a
#else
#define WHEN_CILK_USER_TIMERS(a)
#endif

#define CILK_IM_FREE_MAGIC 0x132DEBFA
#define CILK_CLOSURE_MAGIC 0xDEADBEEF
#define CILK_STACKFRAME_MAGIC 0xBEEF1234

#define CILK_MALLOC_NLISTS 32

/* threshold above which we use slow malloc */
#define CILK_INTERNAL_MALLOC_THRESHOLD 4096
#define CILK_INTERNAL_MALLOC_BUCKETS 9 

/* tell cilk2c not to emit code for linker segments */
#pragma segments -

/*
 * ``parameters'' are set by the global process and they are only read
 * by workers.  It would be nice to map these in a segment which is
 * actually read only.
 */
#define DEFINE_PARAMETER(before,id)  before Cilk_##id##_ro
#define USE_PARAMETER(id) Cilk_##id##_ro
#define INIT_PARAMETER(id, value) USE_PARAMETER(id) = value

#endif				/* _CILK_CONF_H */
